# -*- coding: utf-8 -*-
##############################################################
##
##############################################################

import jngl
from GameObject import GameObject
import random
import copy

RESOURCEWORLD = "img/World/"
class Spike(GameObject):
    def __init__(self, x, y):
        self.img = RESOURCEWORLD + "spike.png"
        self.x = x
        self.y = y
        self.width = jngl.GetWidth(RESOURCEWORLD + "spike.png")
        self.height = jngl.GetHeight(RESOURCEWORLD + "spike.png")
    def drawPreview(self):
        jngl.SetSpriteColor(255, 255, 255, 100)
        self.draw()
        jngl.SetSpriteColor(255, 255, 255)
    def draw(self):
        jngl.Draw(self.img,self.x, self.y)
    def handleCollision(self, player):
        player.kill()

class Key(GameObject):
    def __init__(self, x, y):
        self.img = RESOURCEWORLD + "key.png"
        self.x = x
        self.y = y
        self.width = jngl.GetWidth(self.img)
        self.height = jngl.GetHeight(self.img)
    def drawPreview(self):
        jngl.SetSpriteColor(255, 255, 255, 100)
        self.draw()
        jngl.SetSpriteColor(255, 255, 255)
    def draw(self):
        jngl.Draw(self.img,self.x, self.y)
    def step(self):
        pass
    def handleCollision(self, player):
        if self in self.game.level.objects:
            self.game.level.objects.remove(self)
            player.hasKey = True

class Lock(GameObject):
    def __init__(self, x, y):
        self.img = RESOURCEWORLD + "lock.png"
        self.x = x
        self.y = y
        self.width = jngl.GetWidth(self.img)
        self.height = jngl.GetHeight(self.img)
    def drawPreview(self):
        jngl.SetSpriteColor(255, 255, 255, 100)
        self.draw()
        jngl.SetSpriteColor(255, 255, 255)
    def isBox(self):
        return True
    def draw(self):
        jngl.Draw(self.img,self.x, self.y)
    def handleCollision(self, player):
        if self in self.game.level.objects and player.hasKey:
            self.game.level.objects.remove(self)

class NoJumpArea(GameObject):
    def __init__(self, x, y):
        self.img = RESOURCEWORLD + "yellow_black.png"
        self.x = x
        self.y = y
        self.width = jngl.GetWidth(self.img)
        self.height = jngl.GetHeight(self.img)
    def drawPreview(self):
        jngl.SetSpriteColor(255, 255, 255, 100)
        self.draw()
        jngl.SetSpriteColor(255, 255, 255)
    def draw(self):
        jngl.Draw(self.img, self.x, self.y)
    def handleCollision(self, player):
        player.canJump = False

class SlowArea(GameObject):
    img = RESOURCEWORLD + "slow.png"
    width = jngl.GetWidth(img)
    height = jngl.GetHeight(img)
    def __init__(self, x, y):
        self.x = x
        self.y = y
    def drawPreview(self):
        jngl.SetSpriteColor(255, 255, 255, 100)
        self.draw()
        jngl.SetSpriteColor(255, 255, 255)
    def draw(self):
        jngl.Draw(self.img, self.x, self.y)
    def handleCollision(self, player):
        player.xspeed *= 0.8

class Box(GameObject):
    textureSize = 32
    texture = "boxtex.png"
    gras_texture = "gras.png"
    def __init__(self, x, y, width = 32, height = 32):
        self.x = x
        self.y = y
        self.xspeed = 0
        self.yspeed = 0
        self.width = width
        self.height = height
        if self.width < 0:
            self.width = -self.width
            self.x -= self.width
        if self.height < 0:
            self.height = -self.height
            self.y -= self.height
    def isBox(self):
        return True
    def drawPreview(self):
        self.draw()
    def draw(self):
        jngl.SetColor(200, 200, 200)
        jngl.DrawRect(self.x, self.y, self.width, self.height)
        for x in range(0, self.width, self.textureSize):
            for y in range(0, self.height, self.textureSize):
                jngl.Draw(RESOURCEWORLD + self.texture, self.x + x, self.y + y)
                if y == 0:
                    jngl.Draw(RESOURCEWORLD + self.gras_texture, self.x + x, self.y)

class MovingBox(Box):
    textureSize = 32
    texture = "movingbox.png"
    gras_texture = "gras.png"
    def __init__(self, x, y, width = 32, height = 32, movetoX = 100, movetoY = 100):
        Box.__init__(self,x,y,width,height)
        self.countdown = 0
        self.moveLeft = True
        self.movetoX = movetoX
    def step(self):
        self.countdown -= 1
        if self.countdown < 0:
            self.countdown = self.movetoX
            self.moveLeft = not self.moveLeft
        if self.moveLeft:
            self.x -= 1
        else:
            self.x += 1
    def handleCollision(self, player):
        if self.moveLeft:
            player.x -= 1
        else:
            player.x += 1
                    
                    
class Goal(GameObject):
    def __init__(self, x, y):
        self.img = RESOURCEWORLD + "goal.png"
        self.x = x
        self.y = y
        self.width = jngl.GetWidth(RESOURCEWORLD + "goal.png")
        self.height = jngl.GetHeight(RESOURCEWORLD + "goal.png")
    def drawPreview(self):
        jngl.SetSpriteColor(255, 255, 255, 100)
        self.draw()
        jngl.SetSpriteColor(255, 255, 255)
    def draw(self):
        jngl.Draw(self.img,self.x, self.y)
    def step(self):
        pass
    def handleCollision(self, player):
        player.goal()


class Spawner(GameObject):
    def __init__(self, x, y):
        self.img = RESOURCEWORLD + "spawner_2.png"
        self.x = x
        self.y = y
        self.width = jngl.GetWidth(RESOURCEWORLD + "spawner_2.png")
        self.height = jngl.GetHeight(RESOURCEWORLD + "spawner_2.png")
        self.spawns = True
        self.quantity = 10

    def drawPreview(self):
        jngl.SetSpriteColor(255, 255, 255, 100)
        self.draw()
        jngl.SetSpriteColor(255, 255, 255)
    def draw(self):
        jngl.Draw(self.img,self.x, self.y)
    def handleCollision(self, player):
        if self.spawns:
            self.spawns = False
            player.game.level.objects.remove(self)
        while self.quantity:
            p = copy.copy(player)
            #p.x += random.randint(-10,10)
            p.hasKey = False
            p.xspeed += random.randint(-50,50)/10.0
            p.yspeed = -random.randint(0,100)/10.0
            self.quantity-=1
            self.game.level.somyeols.append(p)

class Firetrap(GameObject):
    img = RESOURCEWORLD + "fire.png"
    width = jngl.GetWidth(img)
    height = 96
    def __init__(self, x, y):
        self.x = x
        self.y = y
        self.active = True
        self.countdown = 100
        self.particles = []
    def step(self):
        self.countdown -= 1
        if self.countdown < 0:
            self.countdown = 100
            self.active = not self.active
        if self.active:
            class Particle:
                def __init__(self, x, y):
                    self.x = x
                    self.y = y
                    self.lifetime = 20
                def step(self):
                    self.y -= 6
                    self.x += random.randint(-5, 5)
                def draw(self):
                    jngl.SetSpriteColor(random.randint(128, 255), random.randint(0, 128), 0, self.lifetime * 12)
                    jngl.DrawScaled(RESOURCEWORLD + "particle.png", self.x, self.y, 1.5)
            self.particles.append(Particle(self.x + 5, self.y + 100))
        for p in self.particles:
            p.step()
            p.lifetime -= 1
            if p.lifetime <= 0:
                self.particles.remove(p)
    def drawPreview(self):
        self.draw()
    def draw(self):
        jngl.Draw(self.img, self.x, self.y + self.height)
        for p in self.particles:
            p.draw()
        jngl.SetSpriteColor(255, 255, 255)
    def handleCollision(self, player):
        if self.active:
            player.kill()

if __name__ == "__main__":
    import main
